<?php
/**
 * Pagadito.
 * 
 * Pagadito Payment Method v1.0.0
 * 
 * Este programa es Software Libre: Usted puede redistribuirlo y/o modificarlo
 * bajo los terminos de la Licencia Publica General Reducida de GNU (GNU Lesser
 * Public Licence), tal como se encuentra publicada por la Free Software
 * Foundation, ya sea por su version 3 o cualquier otra version superior.
 * 
 * Este programa es distribuido en el espiritu de que sea util, pero SIN NINGUNA
 * GARANTIA; sin tampoco garantia implicita de MERCANTIBILIDAD o ADAPTABILIDAD
 * PARA UN USO PARTICULAR. Vea la licencia GNU LGPL para mayores detalles.
 * 
 * @category    Local
 * @package     Pagadito_MPGC
 * @copyright   Copyright (c) 2012 - Pagadito, S. de R.L. (https://www.pagadito.com)
 * @license     http://www.gnu.org/licenses/lgpl.html
 * @author      Pagadito Development Team <developers@pagadito.com>
 */
class Pagadito_MPGC_Model_PaymentLogic extends Mage_Payment_Model_Method_Abstract
{
    protected $_code                    = 'mpgc';
    protected $_isGateway               = true;
    protected $_canAuthorize            = true;
    protected $_canCapture              = true;
    protected $_canCapturePartial       = false;
    protected $_canRefund               = false;
    protected $_canVoid                 = false;
    protected $_canUseInternal          = true;
    protected $_canUseCheckout          = true;
    protected $_canUseForMultishipping  = false;
    protected $_canSaveCc               = false;
    protected $_isInitializeNeeded      = true;
    protected $_supportedCurrencyCodes  = array('USD', 'GTQ', 'HNL', 'NIO', 'CRC', 'PAB', 'DOP');
    private   $_config                  = null;
    private   $_order                   = null;
    public    $_redirectUrl             = '';
    public    $_pgapi                   = null;
    
    /**
     * 
     * @param type $paymentAction
     * @param type $stateObject 
     */
    
    public function initialize($paymentAction, $stateObject)
    {
        /**
         * Cargando configuraciones 
         */
        $this->_config = Mage::getModel('mpgc/config');
        
        /**
         * Cargando Api Pagadito 
         */
        $this->_pgapi = Mage::getModel('mpgc/api');
        
        /**
         * Habilitar Sandbox en Api 
         */
        if($this->_config->sandbox_mode()){
            $this->_pgapi->mode_sandbox_on();
        }
        
        /**
         * Ingresando en la Api el UID y WSK 
         */
        $this->_pgapi->setParams($this->_config->getUID(), $this->_config->getWSK());
        
        
        switch ($paymentAction) {
            case self::ACTION_AUTHORIZE:
            case self::ACTION_AUTHORIZE_CAPTURE:
                $payment = $this->getInfoInstance();
                $this->capture($payment, $payment->getOrder()->getBaseTotalDue());
                break;
            default:
                break;
        }
    }
    
    /**
     * Valida si en el pago puede utilizarse este metodo con la moneda especificada
     * @param String $currencyCode Moneda que se utilizara para el pago
     * @return boolean Puede o no utilizar la moneda especificada
     */
    
    public function canUseForCurrency($currencyCode)
    {
        if(in_array($currencyCode, $this->_supportedCurrencyCodes)){
            return true;
        }
        
        return false;   
    }
    
    /**
     * Cambia en el API el tipo de moneda que esta siendo utilizada
     * @param String $currencyCode Tido de moneda a utilizar
     */
    
    private function setPagaditoCurrency($currencyCode){
        $currency = $this->_config->getCurrency();
        
        if($currency == '---'){
            $currency = strtoupper($currencyCode);
        }
        
        /**
         * Moneda a utilizar 
         */
        switch ($currency) {
            case 'GTQ':
                $this->_pgapi->change_currency_gtq();
                break;
            case 'HNL':
                $this->_pgapi->change_currency_hnl();
                break;
            case 'NIO':
                $this->_pgapi->change_currency_nio();
                break;
            case 'CRC':
                $this->_pgapi->change_currency_crc();
                break;
            case 'PAB':
                $this->_pgapi->change_currency_pab();
                break;
            case 'DOP':
                $this->_pgapi->change_currency_dop();
                break;
            default:
                $this->_pgapi->change_currency_usd();
                break;
        }
    }

    /**
     * Manda la transaccion a Pagadito
     * @param Varien_Object $payment Objeto conteniendo la informacion de pago
     * @param Double $amount Monto a pagar
     * @return type 
     */
    
    public function capture (Varien_Object $payment, $amount)
    {
        $session = Mage::getSingleton('customer/session');
        $write = Mage::getSingleton('core/resource')->getConnection('core_write');
        
        $order = $payment->getOrder();
        
        /**
        * Id de la orden 
        */
        $orderId = $order->getIncrementId();
        $this->createOrderObject($orderId);
        
        $this->setPagaditoCurrency($order->getBaseCurrencyCode());
        
        /**
         * Verificando que si ya se envio la peticion para esta orden a pagadito 
         */
        $url = $this->verifyOrder($orderId);
        if($url !== null){
            $session->setPagaditoQuoteId(0);
            $session->setPagaditoOrderId(0);
            $session->setRedirectUrl($url);
            return;
        }
        
        $session->setPagaditoQuoteId(Mage::getSingleton('checkout/session')->getQuoteId());
        $session->setPagaditoOrderId($orderId);
        
        $this->pendingOrder($orderId);

        /**
         * Crear transaccion en Pagadito 
         */
        if($this->_pgapi->connect()){
            $referencia_pagadito = $this->_pgapi->get_rs_value();
            
            /**
             * Obteniendo los items de la carretilla 
             */
            $items = $order->getAllItems();
            $tmp_amount = 0;
            
            /**
             * Agregando Items a la transaccion 
             */
            foreach ($items as $itemId => $item){
                $tmp_amount = str_replace(',', '', round($item->getPrice(), 2));
                $tmp_amount = (string) number_format($tmp_amount, 2, '.', '');
                $this->_pgapi->add_detail($item->getQtyToInvoice(), $item->getName(), $tmp_amount, "");
            }
            
            /**
             * Agregando shipping si este se encuentra disponible
             */
            if($order->getBaseShippingAmount() > 0){
                $tmp_amount = str_replace(',', '', round($order->getBaseShippingAmount(), 2));
                $tmp_amount = (string) number_format($tmp_amount, 2, '.', '');
                $this->_pgapi->add_detail(1, $order->getShippingDescription(), $tmp_amount, "");
            }
            
            /**
             * Agregando taxes si este se encuentra disponible
             */
            if($order->getBaseTaxAmount() > 0){
                $tmp_amount = str_replace(',', '', round($order->getBaseTaxAmount(), 2));
                $tmp_amount = (string) number_format($tmp_amount, 2, '.', '');
                $this->_pgapi->add_detail(1, $this->_config->getTaxName(), $tmp_amount, "");
            }
            
            /**
             * Aplicando la transaccion 
             */
            //$ern = md5(microtime(true));
            $ern = $orderId;
            $url_retorno = $this->_pgapi->exec_trans($ern);
            if($url_retorno === false){
                /**
                 * Fallo al aplicar el cargo  
                 */
                Mage::getSingleton('core/session')->addError($this->_pgapi->get_rs_code().": Hubo un problema ejecutando la operaci&oacute;n, por favor intente m&aacute;s tarde.");
                Mage::log("PG: Fallo ejecucion de pago para orden '".$orderId."' pagadito retorno '".$this->_pgapi->get_rs_code()."'");
                $this->cancelOrder($orderId);
                $this->_redirectUrl = 'checkout/cart';
                $session->setRedirectUrl('checkout/cart');
            }else{
                /**
                 * Cargo aplicado, redireccionar a pagadito 
                 */
                $sql = "INSERT INTO pagadito_order(order_id, ern, pagadito_ref, url_pagadito, currency, date)
                        VALUES(
                            '".$orderId."',
                            '".$ern."',
                            '".$referencia_pagadito."',
                            '".$url_retorno."',
                            '".$this->_config->getCurrency()."',
                            CURRENT_TIMESTAMP()
                        )";
                $write->query($sql);
                
                $this->_redirectUrl = $url_retorno;
                $session->setRedirectUrl($url_retorno);
            }
        }else{
            Mage::getSingleton('core/session')->addError($this->_pgapi->get_rs_code().": Hubo un problema ejecutando la operaci&oacute;n, por favor intente m&aacute;s tarde.");
            Mage::log("PG: Fallo de conexion con pagadito '".$this->_pgapi->get_rs_code()."'");
            $this->cancelOrder($orderId);
            $this->_redirectUrl = 'checkout/cart';
            $session->setRedirectUrl('checkout/cart');
        }
    }
    
    /**
     * Pendiente a futura implementacion, capture autoriza
     */
    public function authorize (Varien_Object $payment, $amount)
    {
    
    }

    /**
     * Pendiente a futura implementacion
     */
    public function refund (Varien_Object $payment, $amount)
    {
    
    }

    /**
     * Pendiente a futura implementacion
     */
    public function void (Varien_Object $payment)
    {
    
    }

    /**
     * Redireccionar a Pagadito
     * @return String Url del modulo de redireccion a Pagadito
     */
    public function getOrderPlaceRedirectUrl() {        
        return Mage::getUrl('pagadito/mpgc/redirect');
    }
    
    /**
     * Verifica el estado de pago en Pagadito
     * @param String $token Token de ID de Transaccion en Pagadito
     * @param Integer $orderId ID de la orden
     * @return boolean True para operacion completada con exito
     */
    public function processTransacction($token, &$order){
        // Inicializar configuraciones, si estas estan en nulo
        if($this->_config === null){
            $this->initialize(null, null);
        }
        $orderId = null;

        $write = Mage::getSingleton('core/resource')->getConnection('core_write');
        
        $sql = "SELECT order_id, pagadito_ref, ern FROM pagadito_order WHERE pagadito_ref = '".$token."'";
        $rs = $write->query($sql);
        
        if(count($rs) > 0){
            if($this->_pgapi->connect()){
                $row = $rs->fetch();
                
                $order['Id'] = $row['order_id'];
                $order['ref'] = $row['pagadito_ref'];
                $order['ern'] = $row['ern'];
                $order['idt'] = $this->_pgapi->get_rs_value();
                if($this->_pgapi->get_status($token)){
                    if($this->_pgapi->get_rs_status() == 'COMPLETED'){
                        return true;
                    }else{
                        Mage::getSingleton('core/session')->addError("Pago Cancelado.");
                        Mage::log("PG: Transaccion no completada ".$token);
                        return false;
                    }
                }else{
                    Mage::log("PG: Error obteniendo resultado de transac '".$this->_pgapi->get_rs_code()."'");
                }
            }else{
                Mage::log("PG: Fallo de conexion con pagadito '".$this->_pgapi->get_rs_code()."'");
            }            
        }
        
        Mage::getSingleton('core/session')->addError($this->_pgapi->get_rs_code().": Hubo un problema ejecutando la operaci&oacute;n, por favor intente m&aacute;s tarde.");
        
        return false;
    }
    
    /**
     * Cancelar una orden
     * @param Integer $orderId 
     */
    public function cancelOrder($orderId){
        if($this->_order === null){
            $this->createOrderObject($orderId);
        }
        
        $orderOBJ = Mage::getModel('sales/order')->load($this->_order->getId());
        $orderOBJ->setStatus('canceled');
        $orderOBJ->save();
    }
    
    /**
     * Cambiar estado de la orden a pendiente de pago
     * @param Integer $orderId 
     */
    public function pendingOrder($orderId){
        if($this->_order === null){
            $this->createOrderObject($orderId);
        }
        
        $orderOBJ = Mage::getModel('sales/order')->load($this->_order->getId());
        $orderOBJ->setStatus('pending_payment');
        $orderOBJ->save();
    }
    
    /**
     * Cambiar estado de orden a completada
     * @param Integer $orderId 
     */
    public function completeOrder($order){
        if($this->_order === null){
            $this->createOrderObject($order['Id']);
        }
        
        $orderOBJ = Mage::getModel('sales/order')->load($this->_order->getId());
        
        $payment = $this->_order->getPayment();

        $orderOBJ->setStatus('complete')->save();
        
        $payment->setTransactionId($order['idt']);
        $payment->setIsTransactionClosed(0)->save();

    }
    
    /**
     * Crear la orden
     * @param Integer $orderId identificador de la orden
     */
        
    private function createOrderObject($orderId){
        $this->_order = Mage::getModel('sales/order')
                ->loadByIncrementId($orderId);
    }
    
    /**
     * Previene que la carretilla se vacie al encontrase un error al realizar el pago 
     */

    private function restoreCart(){
        $session = Mage::getSingleton('checkout/session');
        $cart = Mage::getSingleton('checkout/cart');

        $items = $this->_order->getItemsCollection();
        foreach ($items as $item) {
            try {
                $cart->addOrderItem($item,$item->getQty());
            }
            catch (Mage_Core_Exception $e){
                if (Mage::getSingleton('checkout/session')->getUseNotice(true)) {
                    Mage::getSingleton('checkout/session')->addNotice($e->getMessage());
                }
                else {
                    Mage::getSingleton('checkout/session')->addError($e->getMessage());
                }
            }
            catch (Exception $e) {
            Mage::getSingleton('checkout/session')->addException($e,
                Mage::helper('checkout')->__('Cannot add the item to shopping cart.')
                );
            }
        }

        $cart->save();
    }
    
    /**
     * Verificar si la orden habia sido registrada previamente
     * @param Integer $orderId Id de la orden a verificar
     * @return String Url de de pago en Pagadito
     */
    private function verifyOrder($orderId){
        $url = null;
        $write = Mage::getSingleton('core/resource')->getConnection('core_write');
        $sql = "SELECT url_pagadito FROM pagadito_order WHERE order_id = '".$orderId."'";
        
        $rs = $write->query($sql);
        if(count($rs) > 0){
            $row = $rs->fetch();
            
            $url = $row['url_pagadito'];
        }
        
        return $url;
    }
}
?>